@@ -86,7 +86,8 @@
             make-cdata cdata-ref cdata-set! cdata&-ref ccast
             pretty-print-ctype
             
-            ctype? ctype-size ctype-align ctype-kind ctype-info ctype-equal?
+            ctype-size ctype-align ctype-kind ctype-info ctype-name
+            ctype? ctype-equal?
             cdata? cdata-bv cdata-ix cdata-ct
             cdata-copy name-ctype
 
@@ -99,16 +100,17 @@
             NULL NULL?
             unwrap-number unwrap-pointer unwrap-array
             ;;
-            mtype-bv-ref mtype-bv-set!
             ;; debug
-            cstruct-fields cstruct-dict
-            cunion-fields cunion-dict
+            cstruct-fields cstruct-select
+            cunion-fields cunion-select
             cfield-name cfield-type cfield-offset
             cbitfield-mtype cbitfield-shift cbitfield-width cbitfield-signed?
             cpointer-type cpointer-mtype
             carray-type carray-length
-            cenum-vald cenum-symd
+            cenum-symf cenum-numf cenum-syml
             cfunction-proc->ptr cfunction-ptr->proc)
+  #:re-export (mtype-bv-ref mtype-bv-set!)
+  #:use-module (ice-9 hash-table)
   #:use-module (ice-9 match)
   #:use-module (ice-9 format)
   #:use-module ((srfi srfi-1) #:select (fold xcons))
@@ -189,29 +191,29 @@
   (type cfield-type)
   (offset cfield-offset))
 
-;; @deftp {Record} <cstruct> fields dict
-;; struct
+;; @deftp {Record} <cstruct> fields select
 ;; @table @var
-;; @item type
+;; @item fields
+;; @item select
 ;; @end table
 ;; @end deftp
 (define-record-type <cstruct>
-  (%make-cstruct fields dict)
+  (%make-cstruct fields select)
   cstruct?
-  (fields cstruct-fields)             ; list of fields (change to vector)
-  (dict cstruct-dict))                ; dict: name => field
+  (fields cstruct-fields)          ; list of fields
+  (select cstruct-select))         ; proc: symbol => field
 
-;; @deftp {Record} <cunion> fields dict
-;; union
+;; @deftp {Record} <cunion> fields select
 ;; @table @var
-;; @item type
+;; @item fields
+;; @item select
 ;; @end table
 ;; @end deftp
 (define-record-type <cunion>
-  (%make-cunion fields dict)
+  (%make-cunion fields select)
   cunion?
-  (fields cunion-fields)           ; list of fields (change to vector)
-  (dict cunion-dict))              ; dict: name => field
+  (fields cunion-fields)           ; list of fields
+  (select cunion-select))          ; proc: symbol => field
 
 ;; @deftp {Record} <carray> type length
 ;; XXX
@@ -225,24 +227,25 @@
   (type carray-type)
   (length carray-length))
 
-;; @deftp {Record} <cenum> mtype symd vald
+;; @deftp {Record} <cenum> mtype symf numf syml
 ;; @table @var
 ;; @item mtype
 ;; machine type to store
-;; @item symd
-;; value to symbol dict (or call it nambynum?)
-;; @item vald
-;; symbol to value (or call it numbynam?)
-;; @table @var
-;; @item type
+;; @item symf
+;; value to symbol proc
+;; @item numf
+;; symbol to value proc
+;; @item syml
+;; list of symbols
 ;; @end table
 ;; @end deftp
 (define-record-type <cenum>
-  (%make-cenum mtype symd vald)
+  (%make-cenum mtype symf numf syml)
   cenum?
   (mtype cenum-mtype)                   ; underlying basic C type
-  (symd cenum-symd)                     ; value -> name
-  (vald cenum-vald))                    ; name -> value
+  (symf cenum-symf)                     ; proc: number -> symbol
+  (numf cenum-numf)                     ; proc: symbol -> number
+  (syml cenum-syml))                    ; list of symbols
 
 ;; @deftp {Record} <cpointer> type mtype
 ;; Once we get to this level, we shouldn't need @code{arch} anymore
@@ -331,65 +334,19 @@
   (unless (cdata? v)
     (error (simple-format #f "~a: expected <cdata>, got ~s" p v))))
 
-(define-syntax be (identifier-syntax (endianness big)))
-(define-syntax le (identifier-syntax (endianness little)))
-
-;; => arch-info
-(define (mtype-bv-ref mtype bv ix)
-  (case mtype
-    ((u8) (bytevector-u8-ref bv ix))
-    ((s8) (bytevector-s8-ref bv ix))
-    ((u16le) (bytevector-u16-ref bv ix le))
-    ((s16le) (bytevector-s16-ref bv ix le))
-    ((u32le) (bytevector-u32-ref bv ix le))
-    ((s32le) (bytevector-s32-ref bv ix le))
-    ((u64le) (bytevector-u64-ref bv ix le))
-    ((s64le) (bytevector-s64-ref bv ix le))
-    ((f32le) (bytevector-ieee-single-ref bv ix le))
-    ((f64le) (bytevector-ieee-double-ref bv ix le))
-    ((u16be) (bytevector-u16-ref bv ix be))
-    ((s16be) (bytevector-s16-ref bv ix be))
-    ((u32be) (bytevector-u32-ref bv ix be))
-    ((s32be) (bytevector-s32-ref bv ix be))
-    ((u64be) (bytevector-u64-ref bv ix be))
-    ((s64be) (bytevector-s64-ref bv ix be))
-    ((f32be) (bytevector-ieee-single-ref bv ix be))
-    ((f64be) (bytevector-ieee-double-ref bv ix be))))
-
-;; => arch-info
-(define (mtype-bv-set! mtype bv ix value)
-  (case mtype
-    ((u8) (bytevector-u8-set! bv ix value))
-    ((s8) (bytevector-s8-set! bv ix value))
-    ((u16le) (bytevector-u16-set! bv ix value le))
-    ((s16le) (bytevector-s16-set! bv ix value le))
-    ((u32le) (bytevector-u32-set! bv ix value le))
-    ((s32le) (bytevector-s32-set! bv ix value le))
-    ((u64le) (bytevector-u64-set! bv ix value le))
-    ((s64le) (bytevector-s64-set! bv ix value le))
-    ((f32le) (bytevector-ieee-single-set! bv ix value le))
-    ((f64le) (bytevector-ieee-double-set! bv ix value le))
-    ((u16be) (bytevector-u16-set! bv ix value be))
-    ((s16be) (bytevector-s16-set! bv ix value be))
-    ((u32be) (bytevector-u32-set! bv ix value be))
-    ((s32be) (bytevector-s32-set! bv ix value be))
-    ((u64be) (bytevector-u64-set! bv ix value be))
-    ((s64be) (bytevector-s64-set! bv ix value be))
-    ((f32be) (bytevector-ieee-single-set! bv ix value be))
-    ((f64be) (bytevector-ieee-double-set! bv ix value be))))
-
-;;.@deffn {Procedure} make-cbase-map arch
-;; where @var{arch} is string or @code{<arch>}
+;;.@deffn {Procedure} make-cbase-map arch => hashq-table
+;; Create a hashq table for @var{arch} mapping base C types to cdata types.
 ;;.@end deffn
-(define (make-cbase-map arch) ;; => ((double . (cbase "double")) ...)
+(define (make-cbase-map arch)
   (define (make-cbase name)
     (let* ((mtype (mtypeof-basetype name))
            (size (sizeof-basetype name))
            (align (alignof-basetype name)))
       (%make-ctype size align 'base mtype #f)))
   (with-arch arch
-    (map (lambda (name) (cons name (make-cbase name)))
-         base-type-symbol-list)))
+    (alist->hashq-table
+     (map (lambda (name) (cons name (make-cbase name)))
+          base-type-symbol-list))))
 
 (define cbase-symbols
   '(s8 u8 s16 s32 s64 i128 u16 u32 u64 u128 f16 f32 f64 f128
@@ -397,7 +354,7 @@
        f16le f32le f64le f128le s16be s32be s64be i128be
        u16be u32be u64be u128be f16be f32be f64be f128be))
 
-;; @deffn {Procedure} cbase name
+;; @deffn {Procedure} cbase name => <ctype>
 ;; Given symbolic @var{name} generate a base ctype.   The name can
 ;; be something like @code{unsigned-int}, @code{double}, or can be a
 ;; @emph{cdata} machine type like @code{u64le}.
@@ -415,12 +372,12 @@
                    (let ((cmap (make-cbase-map arch)))
                      (*cbase-map* (acons arch cmap (*cbase-map*)))
                      cmap))))
-    (or (assq-ref cmap name)
+    (or (hashq-ref cmap name)
         (and (memq name cbase-symbols) name)
         (error "cbase: not found:" name))))
 
-;; @deffn {Procedure} cpointer type
-;; Generate a C pointer type to @var{type}. To reference or de-reference
+;; @deffn {Procedure} cpointer type => <ctype>
+;; Generate a C pointer type for @var{type}. To reference or de-reference
 ;; cdata object see @code{cdata&} and @code{cdata*}.  @var{type} can be
 ;; the symbol @code{void} or a symbolic name used as argument to @code{cbase}.
 ;; @*note: Should we allow @var{type} to be a promise?
@@ -430,8 +387,8 @@
 ;; @end example
 ;; @end deffn
 (define (cpointer type)
-  "- Procedure: cpointer type
-     Generate a C pointer type to TYPE.  To reference or de-reference
+  "- Procedure: cpointer type => <ctype>
+     Generate a C pointer type for TYPE.  To reference or de-reference
      cdata object see ‘cdata&’ and ‘cdata*’.  TYPE can be the symbol
      ‘void’ or a symbolic name used as argument to ‘cbase’.
      note: Should we allow TYPE to be a promise?
@@ -476,7 +433,23 @@
           (acons (cfield-name field) (cfield/moved-offset field offset) seed))
         dict fields))
 
-;; @deffn {Procedure} cstruct fields [packed] => ctype
+;; convert alist to perfect hash lookup : work in progress
+(define (alist->phlkup alist)
+  (define (nextn n) (+ n (/ (if (odd? n) (1+ n) n) 2)))
+  (let loop ((kl '()) (n (length alist)) (mx -1) (mn #xffffffff) (al alist))
+    (if (pair? al)
+        (let ((hv (hash (caar al) n)))
+          (if (memq hv kl)
+              (loop '() (nextn n) -1 #xffffffff alist)
+              (loop (cons hv kl) n (max mx hv) (min mn hv) (cdr al))))
+        (let* ((sz (- mx mn -1))
+               (hv (make-vector sz #f)))
+          (for-each (lambda (k kv) (vector-set! hv (- k mn) (cdr kv))) kl alist)
+          ;; to finish need lambda:
+          (sferr "ral size =~s ph len=~s min=~s\n" (length alist) sz mn)
+          (lambda (sym) (vector-ref hv (- (hash sym n) mn)))))))
+
+;; @deffn {Procedure} cstruct fields [packed] => <ctype>
 ;; Construct a struct ctype with given @var{fields}.  If @var{packed},
 ;; @code{#f} by default, is @code{#t}, create a packed structure.
 ;; @var{fields} is a list with entries of the form @code{(name type)} or
@@ -505,8 +478,9 @@
     ;; cfl: C field list; ral: reified a-list; ssz: struct size;
     ;; sal:struct alignment; sfl: scheme fields
     (if (null? sfl)
-        (%make-ctype (incr-bit-size 0 sal ssz) sal 'struct
-                     (%make-cstruct (reverse cfl) (reverse ral)) #f)
+        (let* ((ral (reverse ral)) (lkup (lambda (sym) (assq-ref ral sym))))
+          (%make-ctype (incr-bit-size 0 sal ssz) sal 'struct
+                       (%make-cstruct (reverse cfl) lkup) #f))
         (match (car sfl)
           ((name type)                  ; non-bitfield
            (let* ((type (cond ((symbol? type) (cbase type))
@@ -552,7 +526,7 @@
            (error "yuck"))))))
 
 
-;; @deffn {Procedure} cunion fields
+;; @deffn {Procedure} cunion fields => <ctype>
 ;; Construct a union ctype with given @var{fields}.
 ;; See @emph{cstruct} for a description of the @var{fields} argument.
 ;; @end deffn
@@ -562,8 +536,9 @@
      description of the FIELDS argument."
   (let loop ((cfl '()) (ral '()) (ssz 0) (sal 0) (sfl fields))
     (if (null? sfl)
-        (%make-ctype (incr-size 0 sal ssz) sal 'union
-                     (%make-cunion (reverse cfl) (reverse ral)) #f)
+        (let* ((ral (reverse ral)) (lkup (lambda (sym) (assq-ref ral sym))))
+          (%make-ctype (incr-size 0 sal ssz) sal 'union
+                       (%make-cunion (reverse cfl) lkup) #f))
         (let* ((name (caar sfl))
                (type (cadar sfl))
                (type (cond ((symbol? type)
@@ -586,7 +561,7 @@
                 (maxi-size fsz fal ssz) (max fal sal) (cdr sfl))))))
 
 
-;; @deffn {Procedure} carray type n
+;; @deffn {Procedure} carray type n => <ctype>
 ;; Create an array of @var{type} with @var{length}.
 ;; If @var{length} is zero, the array length is unbounded: it's length
 ;; can be specified as argument to @code{make-cdata}.
@@ -601,18 +576,20 @@
                'array (%make-carray type n) #f))
 
 
-;; @deffn {Procedure} cenum enum-list [packed]
+;; @deffn {Procedure} cenum enum-list [packed] => <ctype>
 ;; @var{enum-list} is a list of name or name-value pairs
 ;; @example
 ;; (cenum '((a 1) b (c 4))
 ;; @end example
-;; If @var{packed} is @code{#t} the size wil be smallest that can hold it.
+;; If @var{packed} is @code{#t} the size will be smallest that can hold it,
+;; as if defined in C with @code{__attribute__((packed))}.
 ;; @end deffn
 (define* (cenum enum-list #:optional packed)
-  "- Procedure: cenum enum-list [packed]
+  "- Procedure: cenum enum-list [packed] => <ctype>
      ENUM-LIST is a list of name or name-value pairs
           (cenum '((a 1) b (c 4))
-     If PACKED is ‘#t’ the size wil be smallest that can hold it."
+     If PACKED is ‘#t’ the size will be smallest that can hold it, as if
+     defined in C with ‘__attribute__((packed))’."
   (define (short-mtype mn mx)
     (if (< 0 mn)
         (cond
@@ -626,38 +603,54 @@
          ((< mx 32768) (mtypeof-basetype 'uint16_t))
          ((< mx 2147483648) (mtypeof-basetype 'uint32_t))
          (else (mtypeof-basetype 'int)))))
-  (let loop ((nvl '()) (nxt 0) (enl enum-list))
+  (let loop ((snl '()) (nxt 0) (enl enum-list))
     (if (null? enl)
-        (let* ((mx (cdar nvl)) (nvl (reverse nvl)) (mn (cdar nvl))
-               (vnl (map (lambda (p) (cons (cdr p) (car p))) nvl))
-               (mtype (if packed (short-mtype mn mx) (mtypeof-basetype 'int))))
+        (let* ((mx (cdar snl)) (snl (reverse snl)) (mn (cdar snl))
+               (nsl (map (lambda (p) (cons (cdr p) (car p))) snl))
+               (mtype (if packed (short-mtype mn mx) (mtypeof-basetype 'int)))
+               (symf (lambda (num) (assq-ref nsl num)))
+               (numf (lambda (sym) (assq-ref snl sym))))
           (%make-ctype (sizeof-mtype mtype) (alignof-mtype mtype)
-                       'enum (%make-cenum mtype vnl nvl) #f))
+                       'enum (%make-cenum mtype symf numf (map car snl)) #f))
         (match (car enl)
-          (`(,n ,v) (loop (acons n v nvl) (1+ v) (cdr enl)))
-          ((? symbol? n) (loop (acons n nxt nvl) (1+ nxt) (cdr enl)))
-          (((? symbol? n)) (loop (acons n nxt nvl) (1+ nxt) (cdr enl)))
+          (`(,n ,v) (loop (acons n v snl) (1+ v) (cdr enl)))
+          ((? symbol? n) (loop (acons n nxt snl) (1+ nxt) (cdr enl)))
+          (((? symbol? n)) (loop (acons n nxt snl) (1+ nxt) (cdr enl)))
           (_ (error "cenum: bad enum def'n"))))))
 
-;; @deffn {Procedure} cfunction proc->ptr ptr->proc [variadic?]
-;; Generate a C function pointer type.  You must pass the @var{wrapper}
-;; and @var{unwrapper} procedures that convert a pointer to a procedure,
-;; and procedure to pointer, respectively.  The optional argument
-;; @var{#:variadic}, if @code{#t},  indicates the function uses variadic
-;; arguments.  For this case, (to be documented).
+;; @deffn {Procedure} cfunction proc->ptr ptr->proc [variadic?] => <ctype>
+;; Generate a C function type to be used with @code{cpointer}.  You must
+;; pass the @var{wrapper} and @var{unwrapper} procedures that convert a
+;; procedure to a pointer, and pointer to procedure, respectively.  The
+;; optional argument @var{#:variadic}, if @code{#t},  indicates the function
+;; uses variadic arguments.  For this case (I need to add documention).
+;; Here is an example:
+;; @example
+;; (define (f-proc->ptr proc)
+;;   (ffi:procedure->pointer ffi:void proc (list)))
+;; (define (f-ptr->proc fptr)
+;;   (ffi:pointer->procedure ffi:void fptr (list)))
+;; (define ftype (cpointer (cfunction f-proc->ptr f-ptr->proc)))
+;; @end example
 ;; @end deffn
 (define* (cfunction proc->ptr ptr->proc #:optional variadic?)
-  "- Procedure: cfunction proc->ptr ptr->proc [variadic?]
-     Generate a C function pointer type.  You must pass the WRAPPER and
-     UNWRAPPER procedures that convert a pointer to a procedure, and
-     procedure to pointer, respectively.  The optional argument
-     #:VARIADIC, if ‘#t’, indicates the function uses variadic
-     arguments.  For this case, (to be documented)."
+  "- Procedure: cfunction proc->ptr ptr->proc [variadic?] => <ctype>
+     Generate a C function type to be used with ‘cpointer’.  You must
+     pass the WRAPPER and UNWRAPPER procedures that convert a procedure
+     to a pointer, and pointer to procedure, respectively.  The optional
+     argument #:VARIADIC, if ‘#t’, indicates the function uses variadic
+     arguments.  For this case (I need to add documention).  Here is an
+     example:
+          (define (f-proc->ptr proc)
+            (ffi:procedure->pointer ffi:void proc (list)))
+          (define (f-ptr->proc fptr)
+            (ffi:pointer->procedure ffi:void fptr (list)))
+          (define ftype (cpointer (cfunction f-proc->ptr f-ptr->proc)))"
   (let ((type (cbase 'void*)) (mtype (mtypeof-basetype 'void*)))
     (%make-ctype (ctype-size type) (ctype-align type) 'function
                  (%make-cfunction proc->ptr ptr->proc variadic? mtype) #f)))
 
-;; @deffn {Procedure} ctype-detag type ix tag -> type ix
+;; @deffn {Procedure} ctype-detag type ix tag => type ix
 ;; Follows @var{tag}.  For structs and unions, the tag is a symbolic
 ;; field name.  For arrays and pointers, the tag is a non-negative integer.
 ;; An integer tag applied to the pointer increments the pointer by the
@@ -671,11 +664,11 @@
   (let ((ti (ctype-info ct)))
     (case (ctype-kind ct)
       ((struct)
-       (let ((fld (assq-ref (cstruct-dict ti) tag)))
+       (let ((fld ((cstruct-select ti) tag)))
          (unless fld (error "ctype: bad field " tag))
          (values (cfield-type fld) (+ ix (cfield-offset fld)))))
       ((union)
-       (let ((fld (assq-ref (cunion-dict ti) tag)))
+       (let ((fld ((cunion-select ti) tag)))
          (unless fld (error "ctype: bad field " tag))
          (values (cfield-type fld) (+ ix (cfield-offset fld)))))
       ((array)
@@ -717,7 +710,7 @@
           (loop res ct ix (cdr tags))))))))
 
 
-;; @deffn {Procedure} ctype-equal? a b
+;; @deffn {Procedure} ctype-equal? a b => #t|#f
 ;; This predicate assesses equality of it's arguments.
 ;; Two types are considered equal if they have the same size,
 ;; alignment, kind, and eqivalent kind-specific properties.
@@ -766,7 +759,7 @@
                   (ctype-equal? (carray-type a) (carray-type b))))
             ((enum)
              (and (eq? (cenum-mtype a) (cenum-mtype b))
-                  (equal? (cenum-symd a) (cenum-symd b))))
+                  (equal? (cenum-syml a) (cenum-syml b))))
             ((function)
              (and
               (eq? (cfunction-variadic? a) (cfunction-variadic? b))
@@ -783,8 +776,8 @@
      (else (cinfo-equal? (ctype-kind a) (ctype-info a) (ctype-info b))))))
 
 ;; @deffn {Procedure} name-ctype name type -> <ctype>
-;; Add a name to the type.  The name is useful when the type is printed.
-;; This procedure does not mutate: a new type object is created.
+;; Create a new named version of the type.  The name is useful when the type
+;; is printed.  This procedure does not mutate: a new type object is created.
 ;; If a specific type is used by multiple names the names can share
 ;; the underlying type guts.  The following generates two named types.
 ;; @example
@@ -794,21 +787,21 @@
 ;; @end example
 ;; These types are equal:
 ;; @example
-;; (equal? foo_t struct-foo) => #t
+;; (ctype-equal? foo_t struct-foo) => #t
 ;; @end example
 ;; @end deffn
 (define (name-ctype name type)
   "- Procedure: name-ctype name type -> <ctype>
-     Add a name to the type.  The name is useful when the type is
-     printed.  This procedure does not mutate: a new type object is
-     created.  If a specific type is used by multiple names the names
-     can share the underlying type guts.  The following generates two
-     named types.
+     Create a new named version of the type.  The name is useful when
+     the type is printed.  This procedure does not mutate: a new type
+     object is created.  If a specific type is used by multiple names
+     the names can share the underlying type guts.  The following
+     generates two named types.
           (define raw (cstruct '((a 'int) (b 'double))))
           (define foo_t (name-ctype 'foo_t raw))
           (define struct-foo (name-ctype 'struct-foo raw))
      These types are equal:
-          (equal? foo_t struct-foo) => #t"
+          (ctype-equal? foo_t struct-foo) => #t"
   (%make-ctype (ctype-size type) (ctype-align type)
                (ctype-kind type) (ctype-info type)
                name))
@@ -921,8 +914,8 @@
       ((enum)
        (let* ((info (ctype-info ct))
               (mtype (cenum-mtype info))
-              (vnl (cenum-symd info)))
-         (assq-ref vnl (mtype-bv-ref mtype bv ix))))
+              (numf (cenum-numf info)))
+         (numf (mtype-bv-ref mtype bv ix))))
       ((array struct union) (%make-cdata bv ix ct))
       ((function)
        (let* ((ti (ctype-info ct))
@@ -934,13 +927,13 @@
       (else (error "cdata-ref: giving up")))))
 
 ;; @deffn {Procedure} Xcdata-set! bv ix ct value
-;; Reference a deconstructed cdata object, where @var{bv}, @var{ix}
+;; Set the value of a deconstructed cdata object, where @var{bv}, @var{ix}
 ;; and @var{ct} are extracted from a cdata objerct. See @emph{cdata-set!}.
 ;; @end deffn
 (define (Xcdata-set! bv ix ct value)
   "- Procedure: Xcdata-set! bv ix ct value
-     Reference a deconstructed cdata object, where BV, IX and CT are
-     extracted from a cdata objerct.  See _cdata-set!_."
+     Set the value of a deconstructed cdata object, where BV, IX and CT
+     are extracted from a cdata objerct.  See _cdata-set!_."
   (let* ()
     (if (cdata? value)
         (let ((sz (ctype-size ct)))
@@ -983,8 +976,7 @@
               ((integer? value)
                (mtype-bv-set! mtype bv ix value))
               ((symbol? value)
-               (mtype-bv-set! mtype bv ix
-                              (assq-ref (cenum-vald info) value)))
+               (mtype-bv-set! mtype bv ix ((cenum-numf info) value)))
               (else
                (error "cdata-set! bad value arg: ~s" value)))))
           ((array)
@@ -1007,20 +999,15 @@
           ((union) (error "cdata-set!: can't set! union value"))
           (else (error "cdata-set!: bad arg 2" value))))))
 
-;; @deffn {Procedure} cdata-ref data [tag ...]
+;; @deffn {Procedure} cdata-ref data [tag ...] => value
 ;; Return the Scheme (scalar) slot value for selected @var{tag ...} with
 ;; respect to the cdata object @var{data}.
 ;; @example
 ;; (cdata-ref my-struct-value 'a 'b 'c))
 ;; @end example
-;; This procedure returns XXX for cdata kinds @emph{base}, @emph{pointer} and
-;; (in the future) @emph{function}.  Attempting to obtain values for C-type
-;; kinds @emph{struct}, @emph{union}, @emph{array} will result in @code{#f}.
-;; If, in those cases, you would like a cdata then use this:
-;; @example
-;; (or (cdata-ref data tag ...) (cdata-sel data tag ...))
-;; @end example
-;; (Or should we just make this the default behavior?)
+;; This procedure returns Guile values for cdata kinds @emph{base},
+;; @emph{pointer} and @emph{procedure}.   For other cases, a @emph{cdata}
+;; object is returned.  If you always want a cdata object, use @code{cdata-sel}.
 ;; @end deffn
 (define (cdata-ref data . tags)
   "- Procedure: cdata-ref data [tag ...]
@@ -1044,7 +1031,8 @@
 ;; (cdata-set! my-struct-data 42 'a 'b 'c))
 ;; @end example
 ;; If @var{value} is a @code{<cdata>} object copy that, if types match.
-;; @*If @var{value} can be a procedure used to set a cfunction pointer
+;; @*The @var{value} argument can be a procedure when the associated ctype
+;; is a pointer to function.
 ;; value.
 ;; @end deffn
 (define (cdata-set! data value . tags)
@@ -1132,7 +1120,7 @@
   (assert-cdata 'cdata-kind data)
   (ctype-kind (cdata-ct data)))
 
-;; @deffn {Procedure} cdata&-ref data [tag ...]
+;; @deffn {Procedure} cdata&-ref data [tag ...] => value
 ;; Shortcut for @code{(cdata-ref (cdata& data tag ...))}
 ;; This always returns a Guile @emph{pointer}.
 ;; @end deffn
@@ -1146,7 +1134,7 @@
          (addr (+ (pointer-address bptr) (cdata-ix data))))
     (make-pointer addr)))
 
-;; @deffn {Procedure} cdata*-ref data [tag ...]
+;; @deffn {Procedure} cdata*-ref data [tag ...] => value
 ;; Shortcut for @code{(cdata-ref (cdata* data tag ...))}
 ;; @end deffn
 (define (cdata*-ref data . tags)
@@ -1317,10 +1305,6 @@
   (or
    (assq-ref
     `((s8 . ,int8)  (u8 . ,uint8)
-      ;;(s16 . ,int16) (s32 . ,int32) (s64 . ,int64)
-      ;;(u16 . ,uint16) (u32 . ,uint32) (u64 . ,uint64)
-      ;;(f32 . ,float) (f64 . ,double)
-      ;;(s128 . #f) (u128 . #f) (f16 . #f) (f128 . #f)
       (s16le . ,int16) (s32le . ,int32) (s64le . ,int64)
       (u16le . ,uint16) (u32le . ,uint32) (u64le . ,uint64)
       (f32le . ,float) (f64le . ,double)
@@ -1328,14 +1312,26 @@
       (u16be . ,uint16) (u32be . ,uint32) (u64be . ,uint64)
       (f32be . ,float) (f64be . ,double)
       (u128le . #f) (f16le . #f) (f128le . #f) (s128be . #f)
-      (i128le . #f) (u128be . #f) (f16be . #f) (f128be . #f))
+      (i128le . #f) (u128be . #f) (f16be . #f) (f128be . #f)
+      ;;(s16 . ,int16) (s32 . ,int32) (s64 . ,int64)
+      ;;(u16 . ,uint16) (u32 . ,uint32) (u64 . ,uint64)
+      ;;(f32 . ,float) (f64 . ,double)
+      ;;(s128 . #f) (u128 . #f) (f16 . #f) (f128 . #f)
+      )
     mtype)
    (error "mtype->ffi: bad mtype")))
 
-;; @deffn {Procedure} ctype->ffi
-;; doc to come
+;; @deffn {Procedure} ctype->ffi type
+;; Generate a argument spec for Guile's ffi interface.
+;; Example:
+;; @example
+;; (ctype->ffi (cpointer (cbase int))) => '*
+;; @end example
 ;; @end deffn
 (define (ctype->ffi type)
+  "- Procedure: ctype->ffi type
+     Generate a argument spec for Guile's ffi interface.  Example:
+          (ctype->ffi (cpointer (cbase int))) => '*"
   (assert-ctype 'ctype->ffi type)
   (let ((info (ctype-info type)))
     (case (ctype-kind type)
@@ -1344,22 +1340,35 @@
        (make-list (/ (ctype-size type) (ctype-align type))
                   (case (ctype-align type)
                     ((1) int8) ((2) int16) ((4) int32) ((8) int64))))
-      ((pointer array) '*)
-      ((function) (error "ctype->ffi: functions are work to go"))
+      ((pointer array function) '*)
       (else (error "ctype->ffi: unsupported:" (ctype-kind type))))))
 
-;; @deffn {Procedure} unwrap-number
-;; doc to come
+;; @deffn {Procedure} unwrap-number arg
+;; Convert an argument to numeric form for a ffi procedure call.
+;; This will reference a cdata object or pass a number through.
 ;; @end deffn
 (define (unwrap-number arg)
+  "- Procedure: unwrap-number arg
+     Convert an argument to numeric form for a ffi procedure call.  This
+     will reference a cdata object or pass a number through."
   (cond ((number? arg) arg)
         ((cdata? arg) (cdata-ref arg))
         (else (error "unwrap-number: bad arg:" arg))))
 
-;; @deffn {Procedure} unwrap-number
-;; doc to come
+;; @deffn {Procedure} unwrap-pointer arg
+;; Convert an argument to a Guile pointer for a ffi procedure call.
+;; This will reference a cdata object or pass a number through.
+;; If the argument is a function, it will attempt to convert that
+;; to a pointer via @code{procedure->pointer} if given the function
+;; pointer type @var{hint}.
 ;; @end deffn
 (define* (unwrap-pointer arg #:optional hint)
+  "- Procedure: unwrap-pointer arg
+     Convert an argument to a Guile pointer for a ffi procedure call.
+     This will reference a cdata object or pass a number through.  If
+     the argument is a function, it will attempt to convert that to a
+     pointer via ‘procedure->pointer’ if given the function pointer type
+     HINT."
   (cond ((pointer? arg) arg)
         ((string? arg) (string->pointer arg))
         ((cdata? arg) (cdata-ref arg))
@@ -1372,10 +1381,14 @@
            ((cfunction-proc->ptr func) arg)))
         (else (error "unwrap-pointer: bad arg:" arg))))
 
-;; @deffn {Procedure} unwrap-number
-;; doc to come
+;; @deffn {Procedure} unwrap-array arg
+;; This will convert an array to a form suitable to pass to a Guile
+;; ffi procedure.
 ;; @end deffn
 (define (unwrap-array arg)
+  "- Procedure: unwrap-array arg
+     This will convert an array to a form suitable to pass to a Guile
+     ffi procedure."
   (unless
     (cdata? arg)
     (error "unwrap-array: bad arg:" arg))
@@ -1385,7 +1398,14 @@
     (else (error "unwrap-array: bad arg:" arg))))
 
 (define NULL %null-pointer)
+
+;; @deffn {Procedure} NULL? arg
+;; Check if argument is null Guile pointer, or a cdata form of the same.
+;; @end deffn
 (define (NULL? arg)
+  "- Procedure: NULL? arg
+     Check if argument is null Guile pointer, or a cdata form of the
+     same."
   (equal? (if (cdata? arg) (cdata-ref arg) arg) %null-pointer))
 
 ;; --- c99 support -------------------------------------------------------------
